<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2023 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Feb 28, 2023
 */

/**
 * plugin for managing vacation message.
 */
class iw_vacation_message extends rcube_plugin
{
    private const INPUT_TEXTAREA = 'iw-vacation-msg-input';
    private const INPUT_TOGGLE = 'iw-vacation-msg-checkbox';

    // only runs when user selects 'settings'
    public $task = 'settings';

    /**
     * @var rcmail
     */
    private $_rc;

    /**
     * init
     */
    function init()
    {
        $this->_rc = rcmail::get_instance();
        $this->include_stylesheet($this->local_skin_path() . '/style.css');

        $this->add_texts('localization/');
        $this->include_script('client.js');
        $this->add_hook('settings_actions', [$this, 'addToMenu']);
        $this->register_action('plugin.vacationmessage', array($this, 'request_handler'));
        $this->register_action('plugin.vacation-set', array($this, 'setVacationMessage'));
    }

    /**
     * adds menu item
     *
     * @param array $args
     */
    function addToMenu($args)
    {
        $args['actions'][] = [
            'action' => 'plugin.vacationmessage',
            'class'  => 'iw-vacation',
            'label'  => 'vacationmessage',
            'title'  => 'vacationmessage',
            'domain' => 'iw_vacation_message',
        ];

        return $args;
    }

    /**
     * Insert form into default plugin template.
     */
    function request_handler()
    {
        $this->register_handler('plugin.body', array($this, 'form'));
        $this->_rc->output->send('plugin');
    }

    /**
     * creates form for vacation message.
     */
    function form()
    {
        $response_iworx = $this->getVacationMessageArray();
        $status = $response_iworx['on_vacation'] ? 1 : 0;
        $message = $response_iworx['message'];

        $table = new html_table(['cols' => 1, 'class' => 'propform']);

        $input_checkbox = new html_checkbox();
        $table->add('title', html::label(self::INPUT_TOGGLE, $this->gettext('toggle')));
        $table->add(null, $input_checkbox->show(1, [
            'name'  => self::INPUT_TOGGLE,
            'id'    => self::INPUT_TOGGLE,
            'value' => $status,
        ]));

        $props_textarea = [
            'name'         => self::INPUT_TEXTAREA,
            'id'           => self::INPUT_TEXTAREA,
            'rows'         => 5,
            'value'        => $message,
        ];
        $input_msg = new html_textarea($props_textarea);
        $table->add('title', html::label(self::INPUT_TEXTAREA, $this->gettext('vacationmessage')));
        $table->add(null, $input_msg->show());

        $submit_button = $this->_rc->output->button([
            'command' => 'plugin.vacation-set',
            'class'   => 'button mainaction submit',
            'label'   => 'save',
        ]);

        $this->_rc->output->add_gui_object('vacationform', 'vacation-form');

        $form = $this->_rc->output->form_tag(
            [
                'id'     => 'vacation-form',
                'name'   => 'vacation-form',
                'method' => 'post',
                'action' => './?_task=settings&_action=plugin.vacation-set',
            ],
            $table->show()
        );

        $form_buttons = html::p(['class' => 'formbuttons footerleft'], $submit_button);
        $form_div = html::div(['class' => 'boxcontent formcontent'], $form);

        return html::div(['class' => 'box formcontainer scroller'], $form_div . $form_buttons);
    }

    /**
     * set vacation message.
     */
    function setVacationMessage()
    {
        $full_user = $this->getFullUser();
        $message = $this->getInputValue(self::INPUT_TEXTAREA);
        $status = (int) $this->getInputValue(self::INPUT_TOGGLE);
        $file = $this->getBinFilePath('set.sh');
        $cmd = "sudo -u iworx $file $full_user $message $status";

        exec($cmd, $output, $retval);
        $success = $retval === 0;
        $this->setUiMessage($success);

        $this->_rc->output->send();
    }

    /**
     * set ui success/failure message.
     *
     * @param boolean $success
     */
    function setUiMessage($success)
    {
        if ($success) {
            $msg  = $this->gettext('success');
            $this->_rc->output->show_message($msg, 'confirm');
        } else {
            $msg  = $this->gettext('failure');
            $this->_rc->output->show_message($msg, 'error');
        }
    }

    /**
     * Get current vacation message.
     *
     * returns array with keys "message" and "on_vacation"
     *
     * @return array
     */
    function getVacationMessageArray()
    {
        $full_user = $this->getFullUser();
        $file = $this->getBinFilePath('get.sh');
        exec("sudo -u iworx $file $full_user", $output, $retval);

        $json = $output[0];
        return json_decode($json, true);
    }

    /**
     * uses escapeshellarg to sanitize value
     *
     * @return string
     */
    function getFullUser()
    {
        return escapeshellarg($_SESSION['username']);
    }

    /**
     * gets path & uses escapeshellarg
     *
     * might not be needed but will not hurt
     *
     * @return string
     */
    function getBinFilePath($file)
    {
        return escapeshellarg(__DIR__ . "/bin/$file");
    }

    /**
     * Uses escapeshellarg to sanitize input value
     *
     * @return string
     */
    function getInputValue($id)
    {
        $input_value = rcube_utils::get_input_value($id, rcube_utils::INPUT_POST);
        if (is_numeric($input_value)) {
            // no need to use escapeshellarg
            return $input_value;
        }

        return escapeshellarg($input_value);
    }
}
